part of tecfy_corev2_client_package;

class EntityListSearchComponent extends StatefulWidget {
  final Entity entity;
  final String? url;
  final void Function(Map<String, dynamic>?)? onClick;
  final dynamic urlFilterData;
  final Map<String, dynamic> data;
  final PropertyDropdown property;
  final Map<String, dynamic>? rootData;
  final Entity? rootEntity;

  EntityListSearchComponent(
      {required this.property,
      this.onClick,
      required this.entity,
      required this.data,
      this.url,
      this.rootData,
      this.rootEntity,
      this.urlFilterData});
  @override
  _EntityListSearchComponentState createState() =>
      _EntityListSearchComponentState();
}

class _EntityListSearchComponentState
    extends TecfyState<EntityListSearchComponent> {
  late RefreshController _refreshController; // for pull to refrsh
  List<EntityFilter>? _filters;
  List<dynamic>? _data;
  List<dynamic>? _dataFilterd;
  late Entity _entity;
  int _pageIndex = 0;
  bool _noMoreData = false;
  Map<String, dynamic>? parent;

  @override
  void initState() {
    _refreshController = RefreshController(initialRefresh: false);
    _refreshController.footerMode?.addListener(_scrollListner);

    //print(widget.entity);
    _entity = widget.entity;

    //print(entity);
    _getList().then((result) {
      if (result.success) {
        if (parent != null && result.data?.length == 0) {
          widget.onClick?.call(parent);
          return;
        }
        _data = result.data ?? [];
        //An error occurred here when using the filter, so that the data arrives here filtered and does not need to be filtered again
        // _data = result.data
        //         ?.where((d) =>
        //             !widget.property.isFilter ||
        //             MapService.getValue(
        //                     widget.data, widget.property.filterValueField) ==
        //                 MapService.getValue(
        //                     d, widget.property.filterValueField))
        //         .toList() ??
        //     [];
        setState(() {});
      } else
        result.showMessage(context);
    });
    super.initState();
  }

  @override
  dispose() {
    _refreshController.footerMode?.removeListener(_scrollListner);
    _refreshController.dispose();
    super.dispose();
  }

  _scrollListner() async {
    if (kDebugMode)
      print('==========on load ====== ${_refreshController.footerStatus}');
    if (_refreshController.footerStatus != LoadStatus.loading) return;

    _pageIndex++;
    var result = await EntityService.getData(widget.entity,
        pageIndex: _pageIndex, activeOnly: true, filter: _filters);
    var success = result.success && result.data != null;
    if (success) {
      _data?.addAll(result.data ?? []);
      _refreshController.loadComplete();
      _noMoreData = (result.data?.length ?? 0) < EntityService.pageSize;
      _refreshController.refreshCompleted(
          resetFooterState:
              (result.data?.length ?? 0) < EntityService.pageSize);
    } else {
      _refreshController.loadFailed();
    }
    setState(() {});
  }

  Future<OperationResult<List<Map<String, dynamic>>?>> _getList() async {
    try {
      if (kDebugMode)
        print(
            '+++++++++++ ${widget.property.filterField} , ${widget.property.filterValueField}, this is root Data  ${widget.rootData} , this is data ${widget.data}, ${MapService.getValue(widget.data, widget.property.filterValueField)},${MapService.getValue(widget.rootData ?? {}, widget.property.filterValueField)}');
      if (widget.url == null || widget.url!.length == 0) {
        if (!widget.property.isFilter) {
          if (kDebugMode) print('no filter');
          if (_entity.isTree)
            return await EntityService.getData(widget.entity,
                activeOnly: true,
                filterParentId: true,
                parentId: parent?['_id']);
          else
            return await EntityService.getData(
              widget.entity,
              activeOnly: true,
            );
        } else {
          Map<String, dynamic> data = (widget.property.isRootProp == true &&
                  widget.rootData != null &&
                  (widget.rootData?.isNotEmpty ?? false))
              ? widget.rootData ?? widget.data
              : widget.data;
          // Entity? entity =
          //     (widget.property.isRootProp ==
          //                 true &&
          //             widget.rootEntity != null)
          //         ? widget.rootEntity ?? widget.entity
          //         : widget.entity;
          if (MapService.getValue(data, widget.property.filterValueField) !=
              null) {
            _filters = [
              EntityFilter(
                operatorId: 'E',
                value:
                    MapService.getValue(data, widget.property.filterValueField),
                propertyName: widget.property.filterField,
                // property: _entity.properties.firstWhere((element) =>
                //     element.dbName ==
                //     widget.property.filterField
                //         .split(''
                //             '.')
                //         .first),
              )
            ];
            return await EntityService.getData(_entity,
                pageIndex: 0, activeOnly: true, filter: _filters);
          } else if (widget.property.filterQuery.isNotEmpty) {
            _filters = widget.property.filterQuery
                .split('&')
                .map<EntityFilter>((e) {
                  var op = 'E';
                  var opO = '=';
                  if (e.contains('=')) {
                    op = 'E';
                    opO = '=';
                  }
                  if (e.contains('!=')) {
                    op = 'NE';
                    opO = '!=';
                  }
                  if (e.contains('>')) {
                    op = 'G';
                    opO = '>';
                  }
                  if (e.contains('>=')) {
                    op = 'GE';
                    opO = '>=';
                  }
                  if (e.contains('<')) {
                    op = 'L';
                    opO = '<';
                  }
                  if (e.contains('<=')) {
                    op = 'LE';
                    opO = '<=';
                  }
                  var splitValue = e.split(opO);
                  return EntityFilter(
                      operatorId: op,
                      value:
                          EntityService.getValue(widget.data, splitValue.last),
                      property: _entity.properties.firstWhere((element) =>
                          element.dbName ==
                          EntityService.lowerFirst(splitValue.first)));
                })
                .where((e) => e.value != null)
                .toList();
            return await EntityService.getData(_entity,
                filter: _filters, activeOnly: true);
          } else if (widget.property.filterField.isEmpty &&
              widget.property.filterQuery.isEmpty) {
            if (widget.property.filterValueField.toString().contains('.')) {
              var parentPropName =
                  widget.property.filterValueField.split('.')[0];
              var childPropName =
                  widget.property.filterValueField.split('.')[1];

              var parentData = MapService.getValue(
                  widget.data, widget.property.lowerFirst(parentPropName)!);
              if (kDebugMode)
                print('------ Parent Data $parentPropName $parentData');
              var childData =
                  parentData[widget.property.lowerFirst(childPropName)];
              if (kDebugMode)
                print('------ Child Data $childPropName $childData');
              var parentEntity = EntityService.getEntity(parentPropName);
              var childProperty = parentEntity?.properties.firstWhere(
                  (e) => e.dbName.toLowerCase() == childPropName.toLowerCase());
              if (childProperty is PropertyOtherEntityMany) {
                if (childData == null) {
                  _filters = [
                    EntityFilter(
                        operatorId: "E",
                        propertyName: parentEntity!.dbName + "Id",
                        value: parentData["_id"])
                  ];
                  var result = await EntityService.getData(
                      EntityService.getEntity(childProperty.entityName),
                      activeOnly: true,
                      allRecords: true,
                      filter: _filters);
                  if (result.success &&
                      result.data != null &&
                      result.data!.length > 0 &&
                      childProperty.entityName.toLowerCase() !=
                          widget.entity.dbName.toLowerCase()) {
                    if (kDebugMode)
                      print('------------ got data ${result.data!.length}');
                    var childEntity =
                        EntityService.getEntity(childProperty.entityName);
                    var childRelationProperty = childEntity!.properties
                        .firstWhere((e) =>
                            e.propertyType == PropertyTypes.Dropdown &&
                            (e as PropertyDropdown).entityName.toLowerCase() ==
                                widget.entity.dbName.toLowerCase());
                    if (kDebugMode) print(result.data);
                    List<Map<String, dynamic>> lst = [];
                    for (var d in result.data ?? []) {
                      var dd = d[childRelationProperty.dbName.substring(
                          0, childRelationProperty.dbName.indexOf("Id"))];
                      if (dd == null) {
                        dd = await EntityService.getOne(widget.entity.dbName,
                            d[childRelationProperty.dbName]);
                      }
                      if (dd != null) lst.add(dd);
                    }
                    result.data = lst;
                  } else {
                    print('XXXXXXXXXXXXXXXXXXXXXXX not valid data');
                    print(result.success);
                    print(childProperty.entityName);
                    print(widget.entity.dbName);
                    print(result.data);
                  }
                  return result;
                } else {
                  if (kDebugMode) print('--------------- child data not null');
                  var childEntity =
                      EntityService.getEntity(childProperty.entityName);
                  var childRelationProperty = childEntity!.properties
                      .firstWhere((e) =>
                          e.propertyType == PropertyTypes.Dropdown &&
                          (e as PropertyDropdown).entityName.toLowerCase() ==
                              widget.entity.dbName.toLowerCase());
                  List<Map<String, dynamic>> lst = [];
                  if (childData is String) {
                    childData = jsonDecode(childData);
                  }
                  for (var d in childData) {
                    var dd = d[childRelationProperty.dbName.substring(
                        0, childRelationProperty.dbName.indexOf("Id"))];
                    if (dd == null) {
                      dd = await EntityService.getOne(widget.entity.dbName,
                          d[childRelationProperty.dbName]);
                    }
                    if (dd != null) lst.add(dd);
                  }
                  childData = lst;
                  return OperationResult(success: true, data: childData);
                }
              } else {
                if (kDebugMode)
                  print('------------------prop not other many to many!');
              }
            } else {
              if (kDebugMode)
                print('---------------filter field doesn noit has (.)');
              // TODO handel this case
              return OperationResult(success: false);
            }
          }
        }
      } else {
        if (kDebugMode)
          print('================ Filter dropdown from URL =========');
        if (widget.urlFilterData == null)
          return OperationResult(success: false);
        var url = widget.url ?? '';
        for (var p in widget.urlFilterData.keys) {
          if (kDebugMode)
            print('p=$p     value= ${widget.urlFilterData[p].toString()}');
          url = url.replaceAll('{$p}', widget.urlFilterData[p].toString());
        }
        var result = await TecfyBasicApp.apiService.httpGetDynamic(url);
        return OperationResult<List<Map<String, dynamic>>>(
          success: result.success,
          data: result.data
              ?.map<Map<String, dynamic>>((e) => e as Map<String, dynamic>)
              ?.toList(),
        );
      }
      return OperationResult(success: false);
    } catch (err, trace) {
      print('XXXXXXXXXXXXX entity search list -> getList XXXXXXXXXX');
      print(err);
      print(trace);
      return OperationResult(success: false, message: err.toString());
    }
  }

  Future<void> _onRefresh() async {
    _pageIndex = 0;
    var result = await _getList();
    if (result.success) {
      if (parent != null && result.data?.length == 0) {
        widget.onClick?.call(parent);
        return;
      }
      _data = result.data
          ?.where((d) =>
              !widget.property.isFilter ||
              MapService.getValue(
                      widget.data, widget.property.filterValueField) ==
                  d[widget.property.filterField])
          .toList();
      setState(() {});
      _noMoreData = (result.data?.length ?? 0) < EntityService.pageSize;
      _refreshController.refreshCompleted(
          resetFooterState:
              (result.data?.length ?? 0) < EntityService.pageSize);
    } else {
      _refreshController.refreshFailed();
      result.showMessage(context);
    }
  }

  _onLocalSearch(String searchText) async {
    try {
      if (searchText.isEmpty)
        _dataFilterd = null;
      else if (ConnectionService.isOfflineMode) {
        // List<EntityFilter> f = [];
        // for (var prop in entity.properties.where((p) => p.isSearchable)) {
        //   f.add(EntityFilter(
        //       propertyName: prop.dbName,
        //       operatorId: prop.valueType == ValueTypes.Number ? 'E' : 'C',
        //       value: searchText));
        // }
        var d = (await EntityService.getDataSearch(_entity, searchText)).data;
        if (d != null)
          _dataFilterd = d;
        else
          _dataFilterd = [];
      } else {
        _dataFilterd = _data
            ?.where((d) => _entity.gridProperties
                .any((p) => d[p.dbName].toString().contains(searchText)))
            .toList();
      }
      setState(() {});
    } catch (err) {
      print('XXXXXXXXXXX Local Search Error: $err');
    }
  }

  _onServerSearch(String searchText) async {
    try {
      if (searchText.isEmpty)
        _dataFilterd = null;
      else {
        var result = await EntityService.getDataSearch(_entity, searchText);
        if (!result.success) {
          AppNavigator.showMessage(context, 'Server Error', MessageType.error);
        } else
          _dataFilterd = result.data;
      }
      setState(() {});
    } catch (err) {
      print('XXXXXXXXXXX Server Search Error: $err');
    }
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: 300,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
          SearchBox(
              placeholder: AppLocale.translate('Search'),
              onChange: (v) => _onLocalSearch(v),
              onSubmit: (v) => _onServerSearch(v)),
          _getPath(),
          Expanded(
            child: _data == null
                ? Center(child: AppNavigator.getLoadingWidget(50))
                : SmartRefresher(
                    enablePullDown: true,
                    enablePullUp: true,
                    onRefresh: _onRefresh,
                    controller: _refreshController,
                    header: WaterDropHeader(complete: Icon(Icons.done)),
                    footer: CustomFooter(
                      builder: (BuildContext context, LoadStatus? mode) {
                        Widget body;
                        if (mode == LoadStatus.idle && !_noMoreData) {
                          if (LayoutService.isMobile(context)) {
                            body = TecfyText(trans("Pull up load"));
                          } else {
                            body = TextButton(
                                onPressed: () {
                                  _refreshController.requestLoading();
                                  //_scrollListner();
                                },
                                child: TecfyText(trans("click to load mode")));
                          }
                        } else if (mode == LoadStatus.loading) {
                          body = CupertinoActivityIndicator();
                        } else if (mode == LoadStatus.failed) {
                          body = TecfyText("Load Failed!Click retry!");
                        } else if (mode == LoadStatus.canLoading) {
                          body = TecfyText("release to load more");
                        } else {
                          body = TecfyText(trans("No more Data"));
                        }
                        return Container(
                            height: 55.0, child: Center(child: body));
                      },
                    ),
                    child: ListView(
                        children: (_dataFilterd ?? _data ?? [])
                            .map<Widget>((d) => EntityListTileComponent(
                                  _entity,
                                  d,
                                  showUpdateButton: true,
                                  onClick: widget.onClick,
                                  onExpandItem: (c) {
                                    if (_entity.isTree) {
                                      c['parent'] = parent;
                                      parent = c;
                                      _onRefresh();
                                      return;
                                    }
                                    widget.onClick?.call(c);
                                  },
                                ))
                            .toList()),
                  ),
          )
        ],
      ),
    );
  }

  Widget _getPath() {
    if (!_entity.isTree || parent == null) return SizedBox();
    var lst = _addParent([], parent).toList().reversed;
    return Row(
      children: lst
          .map((e) => Row(children: [
                MaterialButton(
                  child: TecfyText(e['name'] ?? e['nameAr'],
                      style: TextStyle(
                          color: lst.last == e
                              ? ThemeService.subColor
                              : ThemeService.mainColor)),
                  onPressed: () {
                    parent = e;
                    _onRefresh();
                  },
                ),
                lst.last == e ? SizedBox() : Icon(Icons.arrow_forward, size: 12)
              ]))
          .toList()
        ..insert(
            0,
            Row(children: [
              IconButton(
                icon: Icon(Icons.home),
                color: ThemeService.mainColor,
                onPressed: () {
                  parent = null;
                  _onRefresh();
                },
              ),
              Icon(Icons.arrow_forward, size: 12)
            ])),
    );
  }

  List<Map<String, dynamic>> _addParent(
      List<Map<String, dynamic>> list, Map<String, dynamic>? node) {
    if (node == null) return list;
    list.add(node);
    if (node['parent'] != null) {
      return _addParent(list, node['parent']);
    }
    return list;
  }
}
